#ifndef USE_OLD_TONEMAP
#define BGE_ACES_TONEMAP
#endif
#ifdef BGE_ACES_TONEMAP
#include "shaders/common/postfx/hdr/ACES.h.hlsl"
#endif

float3 Tonemap_Uchimura_Calc(float3 x, float P, float a, float m, float l, float c, float b) 
{
    // Uchimura 2017, "HDR theory and practice"
    // Math: https://www.desmos.com/calculator/gslcdxvipg
    // Source: https://www.slideshare.net/nikuque/hdr-theory-and-practicce-jp
    
    
    float l0 = ((P - m) * l) / a;
    float L0 = m - m / a;
    float L1 = m + (1.0 - m) / a;
    float S0 = m + l0;
    float S1 = m + a * l0;
    float C2 = (a * P) / (P - S1);
    float CP = -C2 / P;

    float3 w0 = float3(1.0 - smoothstep(0.0, m, x));
    float3 w2 = float3 (step(m + l0, x));
    float3 w1 =float3 (1.0 - w0 - w2);

    float3 T =float3 (m * pow(x / m, c) + b);
    float3 S =float3 (P - (P - S1) * exp(CP * (x - S0)));
    float3 L =float3( m + a * (x - m));

    float3 col=T * w0 + L * w1 + S * w2;
    return col;
}

float3 Tonemap_Uchimura(float3 x) 
{
    const float P = 1.0;  // max display brightness
    const float a = 1.0;  // contrast
    const float m = 0.22; // linear section start
    const float l = 0.28;  // linear section length
    const float c = 1.15; // black
    const float b = 0.0;  // pedestal


    return Tonemap_Uchimura_Calc(x, P, a, m, l, c, b);
}


float3 tonemap(float3 rgb) {
#ifdef BGE_ACES_TONEMAP
    return ACESFitted(rgb);
#else
   return Tonemap_Uchimura(max(rgb, float3(0,0,0)));
#endif
}